       // RESTSRV1: Simple RPG program that supplies a REST web service.
       //
       // This one takes the type of query from the URL and takes additional
       //   parmaeters from the query string e.g. a URL such as:
       //   mysystem.com/restsrv2/quantityquery?partnum=A1234&Quantity=25
       //   The data items being A1234 and 25 and they are associated with names
       //
       // Apache configuration directives shown below were used for the mapping
       //
       // ScriptAlias /restsrv2 /qsys.lib/mylibname.lib/restsrv2.pgm
       // <Directory /qsys.lib/mylibname.lib>
       //   Order Allow,Deny
       //   Allow From All
       // </Directory>

     h Option(*SrcStmt: *NoDebugIO) DftActGrp(*No)
     h BndDir('PARTNER400/CGIDEV2')
     FProduct1  IF   E           K DISK

     d RestSrv2        pr                  ExtPgm('RESTSRV2')
     d RestSrv2        pi

      // pathinfo will contain the portion of the URL that follows the identification
      //   i.e the bit that follows http://xxx.com/restsrv2
     d pathInfo        s            256a   Varying
     d service         s             20a   Varying
     d request         s            256a   Varying
     d savedQuery      s           1000a   Varying
     d inputCount      s             10i 0

       // Variables to contain qury string values
     d partNum         s                   Like(PRODCD)
     d quantity        s                   Like(STOH)

     d QUANTITY_QUERY  c                   'quantityquery'
     d PRICE_QUERY     c                   'pricequery'

      /Free
       // CGIDEV2 Prototypes
      /copy cgidev2/qrpglesrc,prototypeb

       // QUSEC error structure definition
      /copy cgidev2/qrpglesrc,usec


       // Load the xml skeleton into memory - CGIDEV2 identifies the
       //  sections (records) and substitution variable names
       //  second and third parms change the defaults for section markers
       GetHTMLIFS( '/Partner400/RESTSRV2.xml': '<!-- ': ' -->');

       // Write standard response header to buffer
       //  - Note must retain the blank line in the skeleton after content-type
       //  - *off inhibits the output of newline chars
       //  - The '' in the final parm says to output nothing if no value supplied
       //    for one of the substitution variables
       WrtSection('ResponseHeader': *off: '');

       // Use getEnv to grab the back end of the URL
       pathInfo = getEnv('PATH_INFO': qusec );

       // First part of URL identifies the service we want to use
       service = %Subst(pathInfo: 2);

       // inputCount will contain count of total number of different named
       //   variables found in the query string. Not used in this program
       //   but can be useful.
       inputCount = ZhbGetInput(savedQuery: QUSEC);

       // Assume the part number is present and go get it
       partNum = ZhbGetVar('partnum');

       updHTMLvar('partNum': partNum);

       Select;
         When service = QUANTITY_QUERY;
           CheckQuantity(partNum);

         When service = PRICE_QUERY;
           CheckPrice(partNum);

         Other; // Unknown request - build error response
            request = getEnv('QUERY_STRING': qusec );
            updHTMLvar('request': encode(request)); // Encode Query string
            updHTMLvar('service':service);

            WrtSection('ServiceError');
         Endsl;

       // Response complete so add trailer to buffer then return it all to caller
       WrtSection('ResponseTrailer *fini');

       // Use LR On for testing - for production LR off will perform better
       //   but you may need some way of shutting down the program.
       *InLR = *On;

       Return;

      /End-Free

      // CheckQuantity
      //   Retrieves quantity requested (defaults to 1 if not specified)
      //   Checks availability and builds response

     P CheckQuantity   B
     D                 PI
     D  partNum                            Like(PRODCD)

     D quantity        s              5i 0 Inz

      /Free
        If ( ZhbGetVarCnt('quantity') > 0 ); // Was quantity supplied?
          Monitor;
            quantity = %Int( ZhbGetVar('quantity') );
          On-error; // Any conversion error set quantity to default value
            quantity = 1;
          EndMon;
        Else;
          quantity = 1; // No quantity - set to default
        EndIf;

        Chain partNum Product1;

        If %Found(Product1);
          updHTMLvar('quantity': %Char(quantity));
          If STOH >= quantity;
            updHTMLvar('available': %Char(STOH));
            wrtsection('QuantityResponse');
          Else;
            wrtSection('QuantityError');
          EndIf;
        Else;
          wrtsection('PartError');
        EndIf;

        Return;

      /End-Free

     P                 E

      // CheckPrice
      //   Retrieves price of requested part

     P CheckPrice      B
     D                 PI
     D  partNum                            Like(PRODCD)

      /Free

        Chain partNum Product1;

        If %Found(Product1);
          updHTMLvar('price': %Char(SELLPR) );
          wrtsection('PriceResponse');
        Else;
          wrtsection('PartError');
        EndIf;

        Return;

      /End-Free

     P                 E 